@@ -4,8 +4,6 @@ use warnings;
 use base qw 'Blog::Class';
 
 __PACKAGE__->table('entry');
-__PACKAGE__->primary_keys(['entry_id']);
-__PACKAGE__->keys(['uri']);
 __PACKAGE__->has_a(
     user => 'Blog::User',
     { key => 'user_id' }
@@ -15,4 +13,6 @@ __PACKAGE__->has_many(
     { key => 'entry_id' }
 );
 
+sub unique_keys { ['uri'] }
+
 1;
